@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: forms.py 11802 2019-12-17 16:04:02Z Lavender $
+# $Id: forms.py 11888 2020-02-02 14:43:18Z Trista $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Lavender $ (last)
-# $Date: 2019-12-18 01:04:02 +0900 (週三, 18 十二月 2019) $
-# $Revision: 11802 $
+# $Author: Trista $ (last)
+# $Date: 2020-02-02 22:43:18 +0800 (週日, 02 二月 2020) $
+# $Revision: 11888 $
 
 import os
 import re
@@ -48,6 +48,7 @@ from Iuno.shop.models import OrdererAddress, BonusSettings
 catalogueViews = get_class('dashboard.catalogue', 'views')
 
 Product = get_model('catalogue', 'Product')
+UserAddress = get_model('address', 'UserAddress')
 _ProductForm = get_class('dashboard.catalogue.forms', 'ProductForm')
 
 class ProductForm(_ProductForm):
@@ -142,6 +143,12 @@ class UserSearchForm(_UserSearchForm):
         required=False, label=_("Date from"), widget=DatePickerInput)
     dateTo = forms.DateField(
         required=False, label=_("Date to"), widget=DatePickerInput)
+    phone = forms.CharField(
+        required=False, label=_("Phone number"))
+    address = forms.CharField(required=False, label=_("Address"))
+    expenseFrom = forms.FloatField(required=False, label=_("Expense from"))
+    expenseTo = forms.FloatField(required=False, label=_("Expense to"))
+    
     
 _apply_search2 = usersViews.IndexView.apply_search
 def apply_search2(self, queryset):
@@ -156,7 +163,17 @@ def apply_search2(self, queryset):
     
     if data.get('notStaff'):
         queryset = queryset.filter(is_staff=False)
+       
+    if data.get('phone'):
+        queryset = queryset.filter(
+            addresses__phone_number__icontains=data.get('phone')
+        ).order_by('id').distinct()
     
+    if data.get('address'):
+        queryset = queryset.filter(
+            addresses__search_text__icontains=data.get('address')
+        ).order_by('id').distinct()
+
     
     if data['dateFrom'] and data['dateTo']:
         dateTo = datetime_combine(data['dateTo'], datetime.time.max)
@@ -170,6 +187,45 @@ def apply_search2(self, queryset):
         dateTo = datetime_combine(data['dateTo'], datetime.time.max)
         queryset = queryset.filter(date_joined__lt=dateTo)
     
+    Order = get_model('order', 'Order')
+    from django.db.models import Sum
+    
+    if data['expenseFrom'] and data['expenseTo']:
+        userIdList = Order.objects.values('user_id').order_by(
+                'user_id'
+            ).annotate(
+                expense=Sum('total_incl_tax')
+            ).filter(
+                expense__lte=data['expenseTo'], 
+                expense__gte=data['expenseFrom']
+            ).values_list('user_id')
+		queryset = queryset.filter(id__in=userIdList)
+        
+           
+    elif data['expenseFrom']:
+        userIdList = Order.objects.values('user_id').order_by(
+                'user_id'
+            ).annotate(
+                expense=Sum('total_incl_tax')
+            ).filter(
+                expense__gte=data['expenseFrom']
+            ).values_list('user_id')
+		queryset = queryset.filter(id__in=userIdList)
+                
+    elif data['expenseTo']:
+        userIdList = Order.objects.values('user_id').order_by(
+                'user_id'
+            ).annotate(
+                expense=Sum('total_incl_tax')
+            ).filter(
+                expense__lte=data['expenseTo'], 
+            ).values_list('user_id')
+            
+    	queryset = queryset.filter(id__in=userIdList)
+            
+           
+        
+    
     return queryset
 
 usersViews.IndexView.form_class = UserSearchForm
@@ -294,7 +350,7 @@ def get_form(self, **kwargs):
     return form
  
 _getValid =  checkoutViews.ShippingAddressView.form_valid
-UserAddress = get_model('address', 'UserAddress')
+
 def form_valid(self, form):
     rt = _getValid(self, form)
     
@@ -675,9 +731,12 @@ _OrderNoteForm = get_class("dashboard.orders.forms", "OrderNoteForm")
 
 class OrderNoteForm(_OrderNoteForm):
 
+    NORMAL = 'Normal'
+    DISPLAY_FOR_CUSTOMER = 'Display for customer'
+
     CHOICES = [
-        ('Normal', _('Normal')),
-        ('Display for customer', _('Display for customer')),
+        (NORMAL, _(NORMAL)),
+        (DISPLAY_FOR_CUSTOMER, _(DISPLAY_FOR_CUSTOMER)),
     ]
 
     def __init__(self, order, user, *args, **kwargs):
