@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 11852 2020-01-08 13:23:04Z Lavender $
+# $Id: views.py 11881 2020-01-19 12:01:02Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2020-01-08 22:23:04 +0900 (週三, 08 一月 2020) $
-# $Revision: 11852 $ 
+# $Date: 2020-01-19 20:01:02 +0800 (週日, 19 一月 2020) $
+# $Revision: 11881 $ 
 
 import json
 import logging
@@ -34,7 +34,6 @@ from django.contrib.sites.shortcuts import get_current_site
 from django.views.decorators.csrf import csrf_exempt
 from django.utils.decorators import method_decorator
 from django.shortcuts import redirect, get_object_or_404
-from django.core.urlresolvers import reverse
 from django.contrib import messages
 from django.views import generic
 from django.utils.translation import ugettext as _
@@ -53,6 +52,12 @@ from oscar import VERSION
 from Iuppiter.Encoding import utf8
 from Iuppiter.Encryption import Encryptor
 from Iuppiter.DjangoUtil import DJANGO_VERSION
+
+if DJANGO_VERSION <= 20000: # 2.0.0:
+    from django.core.urlresolvers import reverse
+else:
+    from django.urls import reverse
+
 from Iuno.shop import forms
 from Iuno.shop import buildAPIForm
 from Iuno.shop.models import (
@@ -64,6 +69,7 @@ from Iuno.shop.Util import createCheckValue
 _PaymentDetailsView = get_class('checkout.views', 'PaymentDetailsView')
 
 Source = get_model('payment', 'Source')
+RawHTML = get_model('promotions', 'RawHTML')
 SourceType = get_model('payment', 'SourceType')
 Basket = get_model('basket', 'Basket')
 Order = get_model('order', 'Order')
@@ -789,6 +795,8 @@ from oscar.apps.dashboard.orders import views as dashboardOrderViews
 from Iuno.shop.forms import OrderNoteForm
 
 OrderNote = get_model('order', 'OrderNote')
+CommunicationEventType = get_model('customer', 'CommunicationEventType')
+Dispatcher = get_class('customer.utils', 'Dispatcher')
 
 def get_order_note_form(self):
     kwargs = {
@@ -805,7 +813,27 @@ def get_order_note_form(self):
             kwargs['instance'] = note
     return OrderNoteForm(**kwargs)
     
+def save_note(self, request, order):
+    form = self.get_order_note_form()
+    if form.is_valid():
+        note = form.save()
+        if note.note_type == form.DISPLAY_FOR_CUSTOMER:
+            code = 'NOTE_PUBLIC'
+            ctx = {
+                'site': get_current_site(request),
+                'order': order,
+                'note': note,
+            }
+            msg = CommunicationEventType.objects.get_and_render(code, ctx)
+            Dispatcher().dispatch_order_messages(order, msg)
+        messages.success(self.request, _("Note saved"))
+        return self.reload_page(fragment='notes')
+
+    ctx = self.get_context_data(note_form=form, active_tab='notes')
+    return self.render_to_response(ctx)
+    
 dashboardOrderViews.OrderDetailView.get_order_note_form = get_order_note_form
+dashboardOrderViews.OrderDetailView.save_note = save_note
 
 # https://code.nuwainfo.com/trac/Mercurius/ticket/3284#ticket
 try:
@@ -1490,119 +1518,149 @@ class SuccessResponseView2(_PaymentDetailsView):
         self.add_payment_event('Settled', tradeAmt,
                                reference=tradeNo)
 
-class ECPayLogisticreturnView(generic.View):
+class RefundView(generic.View):
 
     http_method_names = ['get', 'post',]
+    
+    def getSource(self, order):
+        if Source.objects.filter(order=order, source_type__name=ECPAY_LOGISTIC).exists():
+            # ecpay logistic
+            isEcpayLogistic = True
+        else:
+            # other
+            isEcpayLogistic = False
+            
+        source = Source.objects.get(order=order)
+        
+        return (source, isEcpayLogistic)
 
     def get(self, request, *args, **kwargs):
         orderNumber = kwargs.get('orderNumber')
         order = get_object_or_404(Order, number=orderNumber)
-        source = Source.objects.get(order=order)
-        tradeNo = source.reference.split("|")
-        if len(tradeNo) == 2: # tradeNo|tradeNo
-            tradeNo = tradeNo[1]
-        else:
-            tradeNo = tradeNo[0]
+        source, isEcpayLogistic = self.getSource(order)
+        
+        if isEcpayLogistic:
+            # ecpay logistic
+            tradeNo = source.reference.split("|")
+            if len(tradeNo) == 2: # tradeNo|tradeNo
+                tradeNo = tradeNo[1]
+            else:
+                tradeNo = tradeNo[0]
 
-        trade = ECPayLogisticTrade.objects.filter(
-                    tradeNo=tradeNo).order_by('-id').first()
+            trade = ECPayLogisticTrade.objects.filter(
+                        tradeNo=tradeNo).order_by('-id').first()
+        else:
+            trade = None
+            
+        info, created = RawHTML.objects.get_or_create(
+            name='Logistic Return',
+            defaults={
+                'body': _('<p>Your refund request has been sent and we will process it as soon as possible.</p>'),
+            }
+        )
         
         return render(
             request, 
-            "shop/payment/EcpayReturn.html",
+            "shop/payment/Refund.html",
             {
                 'order': order,
                 'trade': trade,
+                'info': info,
             }
         )
 
     def post(self, request, *args, **kwargs):
         orderNumber = kwargs.get('orderNumber')
         order = get_object_or_404(Order, number=orderNumber)
-        source = Source.objects.get(order=order)
-        tradeNo = source.reference.split("|")
-        if len(tradeNo) == 2: # tradeNo|tradeNo
-            tradeNo = tradeNo[1]
+        source, isEcpayLogistic = self.getSource(order)
+        
+        if isEcpayLogistic:
+            tradeNo = source.reference.split("|")
+            if len(tradeNo) == 2: # tradeNo|tradeNo
+                tradeNo = tradeNo[1]
+            else:
+                tradeNo = tradeNo[0]
+                
+            trade = ECPayLogisticTrade.objects.filter(
+                        tradeNo=tradeNo).order_by('-id').first()
+           
+            data = {
+                'MerchantID': trade.merchantId,
+                'ServerReplyURL': reverse("ecpayLogisticReply"),
+                'GoodsAmount': trade.tradeAmt,
+                #'GoodsName': "Reference ",
+                'ServiceType': 4, # 退貨不付款
+                'SenderName': trade.receiverName,
+                'SenderPhone': trade.receiverCellPhone,
+                'AllPayLogisticsID': trade.tradeNo,
+            }
+
+            apiUrls = {
+                "FAMI": {
+                    "stage": ("https://logistics-stage.ecpay.com.tw/"
+                             "express/ReturnCVS"),
+                    "production": ("https://logistics-stage.ecpay.com.tw/"
+                                   "express/ReturnCVS"), 
+                },
+                "UNIMART": {
+                    "stage": ("https://logistics-stage.ecpay.com.tw/"
+                              "express/ReturnUniMartCVS"),
+                    "production": ("https://logistics.ecpay.com.tw/"
+                                   "express/ReturnUniMartCVS"), 
+                },
+                "HILIFE": {
+                    "stage": ("https://logistics-stage.ecpay.com.tw/"
+                              "express/ReturnHiLifeCVS"),
+                    "production": ("https://logistics.ecpay.com.tw/"
+                                   "express/ReturnHiLifeCVS"), 
+                },
+            }
+
+            if settings.ECPAY_SANDBOX_MODE:
+                apiUrl = apiUrls[trade.logisticsSubType]["stage"]
+            else:
+                apiUrl = apiUrls[trade.logisticsSubType]["production"]
+
+            data["CheckMacValue"] = createCheckValue(data)
+
+            res = requests.post(apiUrl, data=data)
+
+            if not "ErrorMessage" in res.text:
+                messages.success(self.request, u"成功送出退貨訊息")
+                rtnMerchantTradeNo = res.text.split("|")[0]
+                rtnOrderNo = res.text.split("|")[1]
+
+                newLogisticTrade = ECPayLogisticTrade.objects.create(
+                    tradeNo=trade.tradeNo,
+                    tradeAmt=int(trade.tradeAmt),
+                    merchantId=trade.merchantId,
+                    merchantTradeNo=trade.merchantTradeNo,
+                    rtnCode=100,
+                    rtnMsg=u"已送出退貨資料",
+                    tradeDate=trade.tradeDate,
+                    updateStatusDate=timezone.now(),
+                    receiverName=trade.receiverName,
+                    receiverCellPhone=trade.receiverCellPhone,
+                    receiverEmail=trade.receiverEmail,
+                    logisticsType=trade.logisticsType,
+                    logisticsSubType=trade.logisticsSubType,
+                    paymentNo=trade.paymentNo,
+                    validationNo=trade.validationNo,
+
+                    rtnMerchantTradeNo=rtnMerchantTradeNo,
+                    rtnOrderNo=rtnOrderNo,
+                )
+
+                order.status = settings.OSCAR_ORDER_RETURN_STATUS
+                order.save()
+                return redirect(reverse('refund', kwargs={'orderNumber': order.number}))
+            else:
+                messages.error(self.request, u"送出退貨訊息失敗")
+                return redirect('customer:order-list')
         else:
-            tradeNo = tradeNo[0]
-            
-        trade = ECPayLogisticTrade.objects.filter(
-                    tradeNo=tradeNo).order_by('-id').first()
-       
-        data = {
-            'MerchantID': trade.merchantId,
-            'ServerReplyURL': reverse("ecpayLogisticReply"),
-            'GoodsAmount': trade.tradeAmt,
-            #'GoodsName': "Reference ",
-            'ServiceType': 4, # 退貨不付款
-            'SenderName': trade.receiverName,
-            'SenderPhone': trade.receiverCellPhone,
-            'AllPayLogisticsID': trade.tradeNo,
-        }
-
-        apiUrls = {
-            "FAMI": {
-                "stage": ("https://logistics-stage.ecpay.com.tw/"
-                         "express/ReturnCVS"),
-                "production": ("https://logistics-stage.ecpay.com.tw/"
-                               "express/ReturnCVS"), 
-            },
-            "UNIMART": {
-                "stage": ("https://logistics-stage.ecpay.com.tw/"
-                          "express/ReturnUniMartCVS"),
-                "production": ("https://logistics.ecpay.com.tw/"
-                               "express/ReturnUniMartCVS"), 
-            },
-            "HILIFE": {
-                "stage": ("https://logistics-stage.ecpay.com.tw/"
-                          "express/ReturnHiLifeCVS"),
-                "production": ("https://logistics.ecpay.com.tw/"
-                               "express/ReturnHiLifeCVS"), 
-            },
-        }
-
-        if settings.ECPAY_SANDBOX_MODE:
-            apiUrl = apiUrls[trade.logisticsSubType]["stage"]
-        else:
-            apiUrl = apiUrls[trade.logisticsSubType]["production"]
-
-        data["CheckMacValue"] = createCheckValue(data)
-
-        res = requests.post(apiUrl, data=data)
-
-        if not "ErrorMessage" in res.text:
-            messages.success(self.request, u"成功送出退貨訊息")
-            rtnMerchantTradeNo = res.text.split("|")[0]
-            rtnOrderNo = res.text.split("|")[1]
-
-            newLogisticTrade = ECPayLogisticTrade.objects.create(
-                tradeNo=trade.tradeNo,
-                tradeAmt=int(trade.tradeAmt),
-                merchantId=trade.merchantId,
-                merchantTradeNo=trade.merchantTradeNo,
-                rtnCode=100,
-                rtnMsg=u"已送出退貨資料",
-                tradeDate=trade.tradeDate,
-                updateStatusDate=timezone.now(),
-                receiverName=trade.receiverName,
-                receiverCellPhone=trade.receiverCellPhone,
-                receiverEmail=trade.receiverEmail,
-                logisticsType=trade.logisticsType,
-                logisticsSubType=trade.logisticsSubType,
-                paymentNo=trade.paymentNo,
-                validationNo=trade.validationNo,
-
-                rtnMerchantTradeNo=rtnMerchantTradeNo,
-                rtnOrderNo=rtnOrderNo,
-            )
-
             order.status = settings.OSCAR_ORDER_RETURN_STATUS
             order.save()
-
-            return redirect('customer:order-list')
-        else:
-            messages.error(self.request, u"送出退貨訊息失敗")
-            return redirect('customer:order-list')
+            return redirect(reverse('refund', kwargs={'orderNumber': order.number}))
 
 
 class ECPayServerReplyView(generic.View):
@@ -1756,37 +1814,29 @@ class ATMFillView(generic.View):
 
     def get(self, request, *args, **kwargs):
         order = self.getOrder(request, *args, **kwargs)
-        bank, account = getATMInfo()
+        info = getATMInfo()
         return render(
             request, 
             "shop/payment/FillAccount.html",
             {
                 'order': order,
-                'bank': bank,
-                'account': account,
+                'ATMInfo': info,
             }
         )
         
     def post(self, request, *args, **kwargs):
         order = self.getOrder(request, *args, **kwargs)
         code = request.POST.get('account', None)
-        bank, account = getATMInfo()
-        if code and len(code) == 5: 
-            # 帳號限制要5碼
-            order.setATMCode(code)
-        else:
-            messages.error(
-                request, 
-                _('Please fill in the last five number of your account')
-            )
+        info = getATMInfo()
+        
+        order.setATMCode(code)
         
         return render(
             request, 
             "shop/payment/FillAccount.html",
             {
                 'order': order,
-                'bank': bank,
-                'account': account,
+                'ATMInfo': info,
             }
         )
         
